﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Emgu.CV;
using Emgu.CV.Structure;
using bbiwarg.Graphics;
using bbiwarg.Detectors.Fingers;
using bbiwarg.Utility;

namespace bbiwarg.Images
{
    class DepthImage
    {
        public Image<Gray, byte> Image { get; private set; }
        public Image<Gray, byte> BackgroundMask { get; private set; }
        public int Width { get; private set; }
        public int Height { get; private set; }
        public Int16 MinDepth { get; private set; }
        public Int16 MaxDepth { get; private set; }

        public DepthImage(Image<Gray, Int16> image)
        {
            Width = image.Width;
            Height = image.Height;

            image = image.SmoothMedian(3);

            //threshold min&maxDepth
            MinDepth = findMinDepth(image);
            MaxDepth = (Int16)(MinDepth + 200); // max = minDepth+255 (else it can't fit whole range in byte image)

            //smooth+threshold (dst = (src > (MaxDepth - MinDepth)) ? MaxDepth - MinDepth : src)
            Image = image.Sub(new Gray(MinDepth)).ThresholdTrunc(new Gray(MaxDepth - MinDepth)).Convert<Gray, byte>();
        }

        public Int16 getDepthAt(Point point)
        {
            return getDepthAt(point.X, point.Y);
        }

        public Int16 getDepthAt(int x, int y)
        {
            return (Int16)(MinDepth + Image.Data[y, x, 0]);
        }

        public void setDepthAt(Point point, Int16 depth)
        {
            setDepthAt(point.X, point.Y, depth);
        }

        public void setDepthAt(int x, int y, Int16 depth)
        {
            Image.Data[y, x, 0] = (byte)(depth - MinDepth);
        }

        public float getRelativeDepthAt(Point point)
        {
            return getRelativeDepthAt(point.X, point.Y);
        }

        public float getRelativeDepthAt(int x, int y)
        {
            float minMaxInterval = Math.Max(MaxDepth - MinDepth, 1);
            return (getDepthAt(x, y) - MinDepth) / minMaxInterval;
        }

        private Int16 findMinDepth(Image<Gray, Int16> image)
        {
            // min and max values
            double[] min, max;
            // min and max locations
            Point[] minLoc, maxLoc;

            image.MinMax(out min, out max, out minLoc, out maxLoc);

            return (Int16)min[0];
        }

        public void removeBackground(List<Finger> fingers) {
            BackgroundMask = Image.Copy();
            MCvConnectedComp comp = new MCvConnectedComp();

            foreach (Finger finger in fingers) {
                Vector2D mid = (finger.Tip + finger.Hand) / 2;
                if(BackgroundMask.Data[mid.IntY, mid.IntX,0] != 0)
                    CvInvoke.cvFloodFill(BackgroundMask, mid, new MCvScalar(0), new MCvScalar(5), new MCvScalar(5), out comp, Emgu.CV.CvEnum.CONNECTIVITY.FOUR_CONNECTED, Emgu.CV.CvEnum.FLOODFILL_FLAG.DEFAULT, IntPtr.Zero);
            }

            BackgroundMask = BackgroundMask.ThresholdBinary(new Gray(0), new Gray(255));
            Image = Image.Or(BackgroundMask);

        }
    }
}
